macro_rules! __bool {
( $($whatever:tt)* ) => { ... };
}Expand description
(dev) Always expands to false
Used for conditional expansion of match arms in macros.
__bool! expands to false so that the arm is never executed.
match $something {
$(
_ if __bool!($variable) => unreachable!(),
$match-arm,
)?